#include <string.h> 

#include "stm32f10x.h"
#include "mmc_config.h"
#include "mmc.h"
#include "fat.h"
#include "file.h"
#include "uart.h"

// ******************************************************************************

static void RCC_Configuration(void); 
static void NVIC_Configuration(void);   							   

// ******************************************************************************
int main(void)
{ 
	// use lfn or sfn
	#if (MMC_LFN_SUPPORT == TRUE)
		unsigned char file_name [] = "test_file.txt"; 
	#else 
		unsigned char file_name [] = "test.txt"; 	
	#endif
	
	// we only need a string to write to file, if write is TRUE
	#if (MMC_WRITE == TRUE)
		unsigned char str [] = "Hallo Datei!";
	#endif 

	unsigned long int seek;
	unsigned char c;


	// system clocks config ********************************************
	RCC_Configuration();   

	// nvic config *****************************************************
	NVIC_Configuration();

	// systick config **************************************************
	// callback routine in file: stm32f10x_it.c, function: SysTick_Handler()
	SysTick_Config(720000);	 // 10ms itervall @ 72mhz clock ( number of ticks between two interrupts)

	// uart config *****************************************************
	uart_init(115200); 		// parameter is baud rate

	uputs("Init");

	// sd/mmc config  **************************************************
	if( FALSE == mmc_init() ){
		return FALSE;
	} 

	uputs("...");
			
	// fat config ******************************************************
	if( FALSE == fat_loadFatData() ){
		return FALSE;
	}

	uputs("OK\n");

	#if (MMC_WRITE ==TRUE)		// create and append only if write is TRUE
		// ****************************************************
		// if file exists, it will be opened and then append to it.
/*		if( MMC_FILE_OPENED == ffopen(file_name,'r') ){
			ffseek(file.length);
		   	ffwrites(str);				
			ffwrite(0x0D);		// new line in file
		   	ffwrite(0x0A);
			ffclose();
		}
		*/
		// ***************************************************
		// if the file does not exist, it will be created and written to it.
		if(MMC_FILE_CREATED == ffopen(file_name,'c') ){
			seek = 921600;
			c = 0;
			do{
				ffwrite(c);
				c = (c==126) ? 0:c+1;
			}while(--seek);

			/*ffwrites(str);
			ffwrite(0x0D);		// new line in file
		   	ffwrite(0x0A);*/
			ffclose();
			uputs("\nDone write!");
		} 	
	#endif 

	// ***************************************************
	// read file complete and print via uart!
	if( MMC_FILE_OPENED == ffopen(file_name,'r') ){
		uputs("\nFile exists!");
/*		seek = file.length;
		do{
			uputc( ffread() );						
		}while(--seek);
		ffclose();*/
	}
	
	// retrun from main.c
	return TRUE;	
}



// *******************************************************************************
static void RCC_Configuration(void)
{
  ErrorStatus HSEStartUpStatus;   

  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {	
    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* PLLCLK = 8MHz * 9 = 72 MHz */	 	
	RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    /* Enable PLL */ 
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {
    }

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08)
    {
    }
  }
}

// *******************************************************************************
static void NVIC_Configuration(void)
{
#ifdef  VECT_TAB_RAM  
  /* Set the Vector Table base location at 0x20000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}


// ******************************************************************************
char *ltostr(long num, char *string, unsigned short max_chars, unsigned base)
{
      char remainder;
      int sign = 0;   /* number of digits occupied by the sign. */

      if (base < 2 || base > 36)
            return FALSE;

      if (num < 0)
      {
            sign = 1;
            num = -num;
      }

      string[--max_chars] = '\0';

      for (max_chars--; max_chars > sign && num!=0; max_chars --)
      {
            remainder = (char) (num % base);
            if ( remainder <= 9 )
                  string[max_chars] = remainder + '0';
            else  string[max_chars] = remainder - 10 + 'A';
            num /= base;
      }

      if (sign)
            string[--max_chars] = '-';

      if ( max_chars > 0 )
            memset(string, ' ', max_chars+1);

      return string + max_chars;
}

